/**********
 * Copyright (c) 2003-2004 Greg Parker.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY GREG PARKER ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **********/

#include "includes.h"
#include "about.h"
#include "formutils.h"
#include "data/connectionlist.h"
#include "rsrc/rsrc.h"

#include "mainform.h"


Boolean MainFormHandleEvent(EventPtr e)
{
    ConnectionListHandleEvent(e);

    switch (e->eType) {
    case frmOpenEvent:
        FrmDrawForm(FrmGetActiveForm());
        ConnectionListUpdate();
        return true;

    case winEnterEvent:
        if ((FormPtr)e->data.winEnter.enterWindow == FrmGetFormPtr(MainFormID))
        {
            ConnectionListUpdate();
        }
        return false;

    case menuEvent:
        switch (e->data.menu.itemID) {
        case MenuConnect:
            if (ConnectionListSelectedIndex() == noRecord) {
                FrmPopupForm(LoginFormID);
            } else {
                FrmGotoForm(TerminalFormID);
            }
            return true;
        case MenuHostKeys:
            FrmPopupForm(HostKeysFormID);
            return true;
        case MenuAboutPalmSSH:
            About();
            return true;
        case MenuCredits:
            FrmPopupForm(CreditsFormID);
            return true;
        default:
            return false;
        }

    case ctlSelectEvent:
        switch (e->data.ctlSelect.controlID) {
        case MainConnectButtonID:
            if (ConnectionListSelectedIndex() == noRecord) {
                FrmPopupForm(LoginFormID);
            } else {
                FrmGotoForm(TerminalFormID);
            }
            break;

        case MainNewButtonID:
            FrmPopupForm(LoginFormID);
            break;

        case MainDetailsButtonID:
            if (ConnectionListSelectedIndex() != noRecord) {
                FrmPopupForm(DetailsFormID);
            }
            break;
        }
        return false;

    case frmCloseEvent:
        return false;

    default: 
        return false;
    }
}

