/**********
 * Copyright (c) 2003-2004 Greg Parker.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY GREG PARKER ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **********/

#include "includes.h"
#include "vt100.h"
#include "formutils.h"
#include "rsrc/rsrc.h"
#include "ssh/ssh.h"

#define PUTTY_DO_GLOBALS
#include "putty.h"

static RGBColorType colors[] = {
    {0, 0x22,0x22,0x22}, {0, 0x00,0x00,0x00}, 
    {0, 0xcc,0x00,0x00}, {0, 0xff,0x00,0x00}, 
    {0, 0x00,0xcc,0x00}, {0, 0x00,0xff,0x00}, 
    {0, 0xcc,0xcc,0x00}, {0, 0xff,0xff,0x00}, 
    {0, 0x00,0x00,0xcc}, {0, 0x00,0x00,0xff}, 
    {0, 0xcc,0x00,0xcc}, {0, 0xff,0x00,0xff}, 
    {0, 0x00,0xcc,0xcc}, {0, 0x00,0xff,0xff}, 
    {0, 0xdd,0xdd,0xdd}, {0, 0xff,0xff,0xff}, 
};

extern display *gdd;

static void set_palm_color(int fg_color, int bg_color)
{
    WinSetForeColorRGB(&colors[fg_color], NULL);
    WinSetTextColorRGB(&colors[fg_color], NULL);
    WinSetBackColorRGB(&colors[bg_color], NULL);
}

void WIN_request_resize(int w, int h)
{
    // server-side size change - not supported
    // Disabled with PuTTY's cfg.no_remote_resize
}

/*
 * Draw a line of text in the window, at given character
 * coordinates, in given attributes.
 *
 * We are allowed to fiddle with the contents of `text'.
 */
void WIN_do_text(Context ctx, int x, int y, char *text, int len,
                 unsigned long attr, int lattr)
{
    display *dd = (display *)ctx;

    // fixme implement
    int fg_color;
    int bg_color;
    int font_width = dd->font_width; // hi-res, fixme 
    int font_height = dd->font_height; // hi-res, fixme 

    RectangleType r;

    r.topLeft.x = dd->bounds.topLeft.x + x * font_width;
    r.topLeft.y = dd->bounds.topLeft.y + y * font_height;
    r.extent.x = len * font_width;
    r.extent.y = font_height;


    // fixme cursor foo?

    fg_color = 2 * ((attr & ATTR_FGMASK) >> ATTR_FGSHIFT);
    bg_color = 2 * ((attr & ATTR_BGMASK) >> ATTR_BGSHIFT);

    if (attr & ATTR_REVERSE) {
        int temp = fg_color;
        fg_color = bg_color;
        bg_color = temp;
    }

    // fixme need something similar for underline?
    if (attr & ATTR_BOLD) {
        fg_color++;
    }
    if (attr & ATTR_BLINK) {
        bg_color++;
    }

    // fixme regular=bright, bold&blink = dim
    if (fg_color % 2) fg_color--; else fg_color++;
    if (bg_color % 2) bg_color--; else bg_color++;

    if (attr & (TATTR_ACTCURS | TATTR_PASCURS | TATTR_RIGHTCURS)) {
        int temp = fg_color;
        fg_color = bg_color;
        bg_color = temp;
    }

    // debug_printf("x %d %d %lu %lu %d %s", fg_color, bg_color, attr, lattr, len, text);

    set_palm_color(fg_color, bg_color);

    WinEraseRectangle(&r, 0);
    WinDrawChars(text, len, r.topLeft.x, r.topLeft.y);
}

void WIN_do_cursor(Context ctx, int x, int y, char *text, int len,
                   unsigned long attr, int lattr)
{
    // fixme implement
    // fixme is active/passive/right necessary?
    WIN_do_text(ctx, x, y, text, len, attr, lattr);
}

int WIN_CharWidth(Context ctx, int uc) 
{
    return 1;
    // fixme implement for fonts with double-wide chars
}

void WIN_set_title(char *title)
{
    // fixme change user@host to this title?
}

void WIN_set_icon(char *title)
{
    // fixme probably nowhere to put this
}

void WIN_set_sbar(int total, int start, int page)
{
    Boolean newEnabled;
    Boolean oldEnabled;

    // fixme don't crash if terminal form is not in front
    if (FrmGetActiveFormID() != TerminalFormID) return; 

    // enable or disable up arrows
    newEnabled = (start != 0);
    oldEnabled = PrvGetControlEnabled(TerminalRScrollLineUpButtonID);
    if (newEnabled != oldEnabled) {
        PrvSetControlEnabled(TerminalRScrollLineUpButtonID, newEnabled);
        PrvSetControlEnabled(TerminalRScrollPageUpButtonID, newEnabled);
        if (newEnabled) {
            PrvSetControlGraphics(TerminalRScrollLineUpButtonID, TerminalRScrollLineUpBitmapID, NULL);
            PrvSetControlGraphics(TerminalRScrollPageUpButtonID, TerminalRScrollPageUpBitmapID, NULL);
        } else {
            PrvSetControlGraphics(TerminalRScrollLineUpButtonID, TerminalRScrollLineUpDisabledBitmapID, NULL);
            PrvSetControlGraphics(TerminalRScrollPageUpButtonID, TerminalRScrollPageUpDisabledBitmapID, NULL);
        }
    }

    // enable or disable down arrows
    newEnabled = (total > start + page);
    oldEnabled = PrvGetControlEnabled(TerminalRScrollLineDownButtonID);
    if (newEnabled != oldEnabled) {
        PrvSetControlEnabled(TerminalRScrollLineDownButtonID, newEnabled);
        PrvSetControlEnabled(TerminalRScrollPageDownButtonID, newEnabled);
        if (newEnabled) {
            PrvSetControlGraphics(TerminalRScrollLineDownButtonID, TerminalRScrollLineDownBitmapID, 0);
            PrvSetControlGraphics(TerminalRScrollPageDownButtonID, TerminalRScrollPageDownBitmapID, 0);
        } else {
            PrvSetControlGraphics(TerminalRScrollLineDownButtonID, TerminalRScrollLineDownDisabledBitmapID, 0);
            PrvSetControlGraphics(TerminalRScrollPageDownButtonID, TerminalRScrollPageDownDisabledBitmapID, 0);
        }
    }
}

Context WIN_get_ctx(void)
{
    WinPushDrawState();
    WinSetCoordinateSystem(kCoordinatesDouble);

    FntSetFont(gdd->font);

    return gdd;
}

void WIN_free_ctx(Context ctx)
{
    WinPopDrawState();
}

void WIN_palette_set(int n, int r, int g, int b)
{
    // fixme Linux foo
}

void WIN_palette_reset(void)
{
    // fixme
}

void WIN_write_aclip(char *data, UInt16 len, int must_deselect)
{
    // fixme copy&paste implementation
}

/*
 * Note: unlike write_aclip() this will not append a nul.
 */
void WIN_write_clip(wchar_t * data, UInt16 len, int must_deselect)
{
    // wchar_t bad
    char *newdata;
    unsigned int i;
    // last byte of data is nul char - skip it
    // convert newlines - Palm uses LF
    len--;
    newdata = smalloc(len);
    for (i = 0; i < len; i++) {
        if (data[i] == 13) newdata[i] = 10;
        else newdata[i] = (char)data[i];
    }

    ClipboardAddItem(clipboardText, newdata, len);
    sfree(newdata);
}

void WIN_get_clip(wchar_t ** p, UInt16 *len)
{
    MemHandle clipH;
    char *clipP;
    static wchar_t *wclipP = NULL;

    if (p) {
        clipH = ClipboardGetItem(clipboardText, len);
        if (clipH) {
            UInt16 i;
            clipP = MemHandleLock(clipH);
            wclipP = smalloc(*len * sizeof(wchar_t));
            for (i = 0; i < *len; i++) {
                wclipP[i] = (wchar_t)clipP[i];
            }
            *p = wclipP;
            MemHandleUnlock(clipH);
        } else {
            // no clipboard to paste
            *p = NULL;
            wclipP = NULL;
        }
    } else {
        // clear previously retrieved clip
        if (wclipP) {
            sfree(wclipP);
            wclipP = NULL;
        }
    }
}

void WIN_set_raw_mouse_mode(int activate)
{
    // fixme does Palm raw mouse mode mean anything
    // disable with cfg.no_mouse_rep
}

Mouse_Button WIN_translate_button(Mouse_Button button)
{
    // transform left/middle/right to select/extend/paste
    // Only one "mouse button" on Palm, and it selects.

    return MBT_SELECT;
}

void WIN_beep(int mode)
{
    // fixme audible or visual beep
    // fixme mode is win32-specific?
}

void WIN_sys_cursor(int x, int y)
{
    // no system cursor
    // Only used on Windows for the benefit of screen-readers
}

void WIN_set_iconic(int iconic)
{
    // server-side iconification request
    // No such thing as an iconic Palm window
}

void WIN_move_window(int x, int y)
{
    // server-side move request
    // No such thing as a moved Palm window
}

void WIN_set_zorder(int top)
{
    // server-side restack request
    // No such thing as Palm z-order
}

void WIN_refresh_window(void)
{
    // server-side redraw request
    // fixme
}

void WIN_set_zoomed(int zoomed)
{
    // server-side window zoom request
    // No such thing as a Palm zoomed window
}

int WIN_is_iconic(void)
{
    // No such thing as an iconic Palm window
    return 0;
}

void WIN_get_window_pos(int *x, int *y)
{
    // fixme in pixels?
    *x = 0;
    *y = 0;
}

void WIN_get_window_pixels(int *x, int *y)
{
    // fixme
    *x = 160;
    *y = 120;
}

char *WIN_get_window_title(int icon)
{
    // fixme
    return "fixme window title";
}


unsigned long WIN_GetTickCount(void)
{
    return TimGetTicks() * SysTicksPerSecond() / 1000L;
}


unsigned long WIN_GetCaretBlinkTime(void)
{
    return 500; // fixme
}


// fixme no printing
printer_enum *printer_start_enum(int *nprinters)
{
    *nprinters = 0;
    return 0;
}
char *printer_get_name(printer_enum *a, int b)
{
    return NULL;
}
void printer_finish_enum(printer_enum *c)
{
}
printer_job *printer_start_job(char *printer)
{
    return NULL;
}
void printer_job_data(printer_job *d, void *e, int f)
{
}
void printer_finish_job(printer_job *g)
{
}


void ldisc_send(char *buf, int len, int interactive)
{
    if (buf && len > 0) {
        ssh_bytes(ssh_get_current(), buf, len);
    }
}
